#!/usr/local/bin/perl

# show available ip space

use lib ('/export/web/ipanalyze.example.com/data/lib');

use strict;
use DBI;
use Net::CIDR::Lite;
use IPAnalyze::IPmath;

my $DEBUG = 0;
my $arin = 1 if($ARGV[0] eq '-f'); # set to see as far as ARIN is conserned

my %cfg;
open(CONFIG, '/export/web/ipanalyze.example.com/data/ipanalyze.conf') || die "cannot open ipanalyze.conf: $!\n";
while(<CONFIG>){
   s/^(.*)\s*#.*/$1/;
   next if(/^$/);
   chomp;
   if(/^(\S+)\s*=>\s?(.+)/){
      $cfg{$1} = $2;
   }
}
close CONFIG;

my $ipmath = IPAnalyze::IPmath->new;
my $dbh = DBI->connect($cfg{d_dsn}, $cfg{d_dbun}, $cfg{d_dbpw}, {RaiseError => 1});

my %cidr_pool;
my $sql = 'SELECT name FROM areas';
my $sth = $dbh->prepare($sql);
$sth->execute;
while(my $row = $sth->fetchrow_arrayref){
	$cidr_pool{$row->[0]} = Net::CIDR::Lite->new;
}

$sql = 'SELECT netblock,cidr,areas.name FROM parent_networks,areas';
$sql .= ' WHERE parent_networks.area = areas.id';
$sth = $dbh->prepare($sql);
$sth->execute;
while(my $row = $sth->fetchrow_arrayref){
	$cidr_pool{$row->[2]}->add("$row->[0]/$row->[1]");
}

my (%gp,%p);
if($arin){
	my $sql = 'SELECT id,netblock,cidr FROM grandparent_networks';
	my $sth = $dbh->prepare($sql);
	$sth->execute;
	while(my $row = $sth->fetchrow_arrayref){
		$gp{$row->[0]} = { netblock => $row->[1], cidr => $row->[2] };
	}
	if($DEBUG){
		foreach my $key (keys %gp){
			print "g{$key} = netblock => $gp{$key}{netblock}  cidr => $gp{$key}{cidr}\n";
		}
	}
}else{
	my $sql = 'SELECT id,parent_id,netblock,cidr FROM parent_networks';
	$sql .= ' WHERE allow_use = 1';
	my $sth = $dbh->prepare($sql);
	$sth->execute;
	while(my $row = $sth->fetchrow_arrayref){
		$p{$row->[0]} = { parent_id => $row->[1], netblock => $row->[2], cidr => $row->[3] };
	}

	if($DEBUG){
		foreach my $key (keys %p){
			print "p{$key} = parent_id => $p{$key}{parent_id} - netblock => $p{$key}{netblock} - cidr => $p{$key}{cidr}\n";
		}
	}
}

my $cidr = Net::CIDR::Lite->new;
my $cidr_avail = Net::CIDR::Lite->new;

$sql = 'SELECT netblock,cidr FROM data WHERE netblock IS NOT NULL';
$sth = $dbh->prepare($sql);
$sth->execute;
while(my $row = $sth->fetchrow_arrayref){
	$cidr->add("$row->[0]/$row->[1]");
}

my %hash = ($arin) ? %gp : %p;
foreach my $key (keys %hash){
	# gives all the /30s in the parent 
	my @netblocks = $ipmath->get_netblocks($hash{$key}{netblock},$hash{$key}{cidr},30);
	foreach my $netblock (@netblocks){
		unless($cidr->find($netblock)){
			# the netblock is avail
			$cidr_avail->add("${netblock}/30");
		}
	}
}

foreach my $avail ($cidr_avail->list){
	print "AVAIL: $avail - AREA: ";
	my $f;
	my ($netblock) = $avail =~ /^(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/;
	foreach my $area (keys %cidr_pool){
		if($cidr_pool{$area}->find($netblock)){
			print "$area\n";
			$f=1;
			last;
		}
	}
	print "UNKNOWN\n" unless($f);
}
